// ---------------------------------------------------------------------------------
//
//   A "35mm look" shader for video to be used in Avery Lee's Virtualdub 
//   GPU Shader filter. 2007 Jukka Korhonen from the original code by Avery Lee.
//
// ---------------------------------------------------------------------------------

texture vd_srctexture;
float4 vd_texsize;

sampler src = sampler_state {
	Texture = <vd_srctexture>;
	AddressU = clamp;
	AddressV = clamp;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
};

void VS(
	float4 pos : POSITION,
	float2 t0 : TEXCOORD0,
	out float4 oPos : POSITION,
	out float2 oT0 : TEXCOORD0
)
{
	oPos = pos;
	oT0 = t0;
}

float4 PS(float2 uv : TEXCOORD0) : COLOR0 {
   float4 B = tex2D(src, uv);
   float4 G = B;									// Store the original image.
   float4 H = 0.5;								// The strange 0.5

	B = lerp(H, B, 0.6); 	// Main input contrast.
	
	B = pow(B, 1.00);  	// Main Gamma correction.
											// inverse values, ie.0.45 = Gamma of 2.2 
											// Note: don't overdo it, or you'll 
											// have bigger hue-shifts than you can 
											// handle.

   float A = dot(float3(1.0/3.0, 1.0/3.0, 1.0/3.0), B.rgb); 	// A copy of the original
											// image is created in
											// equally weighted grayscale.
   
   float4 D = A; 									// Strange transformations. 

   float a = 6;			// Strength of the S curve in Red.
   float b = 2;			// Strength of the S curve in Green.
   
   float y = 1 / (1 + exp(a / 2));						// This creates the sigmoid ("S") curve.
   float z = 1 / (1 + exp(b / 2));         
   
   float4 C;			

	D = lerp(H, D, 1.0);							// Contrast for the gray layer.
	B = lerp(H, B, 1.0);   							// Contrast for the background layer. 

	D.r = pow(D.r, 1.0);							// This is an optional gamma correction for Red and
	D.g = pow(D.g, 1.0);   							// green component of the gray layer. 

	D.r = (1 /(1 + exp(- a * (D.r - .5))) - y)/(1 - 2 * y);	// Red S curve.
	D.g = (1 /(1 + exp(- b * (D.g - .5))) - z)/(1 - 2 * z);	// Green S curve. Both in 
											// the gray layer.
	float E = dot(float3(1.0/3.0, 1.0/3.0, 1.0/3.0), D.rgb);	

   	if (E < 0.5)		
		C = 2 * D * B;	
   
   	else
		C = 1 - 2 * (1 - D) * (1 - B);

	float4 F = lerp(G, C, 1.0);	// Opacity of the effect.

   return F;
}

technique {
	pass {
		VertexShader = compile vs_2_a VS();
		PixelShader = compile ps_2_a PS();
	}
}
